=head1 NAME

SUGR::AffineCamera -- class for an affine camera

NOT WORKING YET!

=head1 NOT DOCUMENTED YET

=head1 BUGS

Probably many, but not documented yet.

=head1 SEE ALSO

SUGR(3), SUGR::Element(3)

=head1 AUTHOR

Stephan Heuel
(stephan@heuel.org)

=head1 COPYRIGHT

Copyright (C) 2000/2001 Stephan Heuel and Institute for Photogrammetry,
University of Bonn. All rights reserved. There is no warranty. You are
allowed to redistribute this software / documentation under certain
conditions, see the GNU Public License, GPL for details.

=cut

package SUGR::AffineCamera;

use strict;
use Carp;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
use SUGR::TransformationNew;

$VERSION = SUGR->VERSION;

@ISA = qw(Exporter SUGR::TransformationNew);

@EXPORT = qw(sugr_AffineCamera); # symbols to autoexport
@EXPORT_OK = qw(); #symbols to export on request
%EXPORT_TAGS = ( );

use SUGR::Basic qw/:ALL/;

# the following two constructor helper functions may be rewritten less
# redundantly, since they actually only check dimensions, but what the
# heck!

# constructor with no covariance matrix
sub ConstructFromMatrix { 
  ConstructFromMatrixMatrix(@_);
}

# this is the actual constructor !
sub ConstructFromMatrixMatrix {
  my ($matrix,$cov) = @_;
  croak("no transformation matrix given") if !defined($matrix);

  # check dimensions
  my ($r,$c) = $matrix->mdims;
  if ($r*$c==8) {
    $matrix = $matrix->reshape(2,4);
  }
  else {
    croak("first argument should be a (2,4) matrix, is $matrix");
  }

  my $self = defined($cov) ?
    SUGR::TransformationNew::ConstructFromMatrixMatrix($matrix,$cov) :
	SUGR::TransformationNew::ConstructFromMatrix($matrix);

  # this is so that I don't have to compute the projection matrix for
  # lines over and over again.
#  ($self->{vec_line},$self->{cov_line}) = LineProjection($self->{vec},$self->{cov});

  return $self;
}

sub type {
  "AffineCamera";
}




my @euclid_indices = [0,5,10];
my @homo_indices = [1..4,6..9,11];

sub euclid_scale {
  my ($self) = shift;
  return sqrt(PDL::inner($self->vec->dice(@euclid_indices)->clump(-1),
			 $self->vec->dice(@euclid_indices)->clump(-1)));
}

# needed for euclidize
sub Jacobian_norm_euclid {
  my ($self) = @_;
  die "SUGR::AffineCamera::Jacobian_norm_euclid don't use this, I don't know what happens here!\n";
  return undef;
}

sub scale_euclidean_part {
  my ($self,$scalar) = @_;

  # generate Jacobian
  my $M = SUGR::Element::Identity(12);
  ($_ = $M->diagonal(0,1)->dice(@euclid_indices)) .= $scalar;

  $self->setvec($M x $self->vec);
  $self->setcov($M x $self->cov x ~$M) if defined($self->cov);
  return $self;
}

sub norm_of_euclidean_part {
  my $self = shift;
  sqrt(PDL::sum($self->vec->dice(@euclid_indices)**2));
}

sub norm_of_homogeneous_part {
  my $self = shift;
  PDL::sum(abs($self->vec->dice(@homo_indices)));
}

sub Constraint_Euclidean {
  my ($P) = @_;
  $P = (ref($P) =~ /SUGR/) ? $P->vec : $P;
  my $ret =  vpdl([ 0.5*PDL::sum($P->dice(@homo_indices)**2) - 0.5 ]);
  return $ret;
}
sub Jacobian_Constraint_Euclidean {
  my ($P) = @_;
  my $v = (ref($P) =~ /SUGR/) ? $P->vec : $P;
  my $J = vzeroes(12);
  ($_ = $J->dice(@homo_indices)) .= $v->dice(@homo_indices);
  
  return ~$J;
}

sub Constraint_Sphere {
  my ($P,$squared_radius) = @_;
  $P = (ref($P) =~ /SUGR/) ? $P->vec : $P;
  $squared_radius ||=1;
  return vpdl([ 0.5*PDL::sum($P**2) - 0.5*$squared_radius]);
}
sub Jacobian_Constraint_Sphere {
  my ($P) = @_;
  $P = (ref($P) =~ /SUGR/) ? $P->vec : $P;
  ~$P;
}

return 1 if caller;

#################################################################
# Test code
