=head1 NAME

SUGR::Homography2D -- class for any projective transformation in the plane

=head1 NOT DOCUMENTED YET

=head1 BUGS

Probably many, but not documented yet.

=head1 SEE ALSO

SUGR(3), SUGR::Element(3)

=head1 AUTHOR

Stephan Heuel
(stephan@heuel.org)

=head1 COPYRIGHT

Copyright (C) 2000/2001 Stephan Heuel and Institute for Photogrammetry,
University of Bonn. All rights reserved. There is no warranty. You are
allowed to redistribute this software / documentation under certain
conditions, see the GNU Public License, GPL for details.

=cut

package SUGR::Homography2D;

use strict;
use Carp;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);


use SUGR::TransformationNew;

$VERSION = SUGR->VERSION;

@ISA = qw(Exporter SUGR::TransformationNew);

@EXPORT = qw(sugr_Homography2D); # symbols to autoexport
@EXPORT_OK = qw(); #symbols to export on request
%EXPORT_TAGS = ( );

# this is a constructor that can be called like it is
sub sugr_Homography2D { SUGR::Homography2D->new(@_) }

return 1 if caller;

# the following two constructor helper functions may be rewritten less
# redundantly, since they actually only check dimensions, but what the
# heck!

# constructor with no covariance matrix
sub ConstructFromMatrix { 
  my ($matrix) = @_;
  croak("no transformation matrix given") if !defined($matrix);

  # check dimensions
  my ($r,$c) = $matrix->mdims;
  croak("first argument should be a (3,3) matrix, is $matrix") 
    if (($r!=3 && $c!=3));
  
  SUGR::TransformationNew::ConstructFromMatrix($matrix);
}

# this is the actual constructor !
sub ConstructFromMatrixMatrix {
  my ($matrix,$cov) = @_;
  croak("no transformation matrix given") if !defined($matrix);

  # check dimensions
  my ($r,$c) = $matrix->mdims;
  croak("first argument should be a (3,3) matrix, is $matrix") 
    if (($r!=3 && $c!=3));

  my $self = SUGR::TransformationNew::ConstructFromMatrixMatrix($matrix,$cov);
  return $self;
}


################################################################################
# Test code
use PDL::Matrix;

my $T = sugr_Homography2D(msequence(3,3),
			  mones(9,9));

print $T;

__END__
