=head1 NAME

SUGR::Projection -- class for projections of points and lines from 3D to 2D

(don't use this anymore, use SUGR::ProjecticeCamera !)

=head1 VERSION

  This document refers to version 0.01

=head1 SYNOPSIS


=head1 DESCRIPTION

=head2 Overview

=head2 Constructurs and initializaton

=head2 Class and object methods

=head2 Something else?!

=head2 Literature

  see Note 'Homogeneous Transformation of Points, Planes and Lines in
  3D' by Wolfgang Frstner, 2000

=head1 DIAGNOSTICS

=over 4

=item "error messages?!"

=back

=head1 BUGS

=head1 TODO

=head1 FILES

=head1 SEE ALSO

=head1 AUTHOR

Stephan Heuel
(stephan@heuel.org)

=head1 COPYRIGHT

Copyright (C) 2000 Stephan Heuel All rights reserved. There is no
warranty. You are allowed to redistribute this software /
documentation under certain conditions. For details, see the file
COPYING in the PDL distribution. If this file is separated from the
PDL distribution, the copyright notice should be included in the file.

=cut

package SUGR::Projection;

use strict;
use Carp;
use vars qw(@EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
use SUGR::Element;
our @ISA = qw(SUGR::Element Exporter);
# use PDL;
# use PDL::Math;
# use PDL::Matrix;
#use SUGR::Line;

$VERSION = 0.01;

@EXPORT = qw(sugr_Projection); # symbols to autoexport
@EXPORT_OK = qw(); #symbols to export on request
%EXPORT_TAGS = ( );


#
# this is a constructor that can be called like it is
sub sugr_Projection { SUGR::Projection->new(@_) }

# constructor with no covariance matrix
sub ConstructFromMatrix { 
  my ($matrix) = @_;
  croak("no parameter vector given") if !defined($matrix);
  my ($r,$c) = $matrix->mdims;

  # currently only point projection matrices can be used as input
  croak("first argument should be a (3,4) matrix or a (12,1) vector, is $matrix") 
    if (($r!=3 && $c!=4) && ($r!=12 && $c!=1));
  
  my $self = SUGR::Element::ConstructFromMatrix($matrix->reshape(12,1));

  ($self->{vec_line}) = LineProjection($self->{vec});
  return $self;
}

# this is the actual constructor !
sub ConstructFromMatrixMatrix {
  my ($matrix,$cov) = @_;
  croak("no projection matrix given") if !defined($matrix);
  croak("transformation is no PDL::Matrix") if ref($matrix) ne "PDL::Matrix";

  my ($r,$c) = $matrix->mdims;

  croak("first argument should be a (3,4) matrix or a (12,1) vector, is $matrix") 
    if (($r!=3 && $c!=4) && ($r!=12 && $c!=1));
  croak("covariance matrix is no PDL::Matrix") if ref($cov) ne "PDL::Matrix";

  my $self = SUGR::Element::ConstructFromMatrixMatrix($matrix->reshape(12,1),$cov);

  # this is so that I don't have to compute this over and over again.
  ($self->{vec_line},$self->{cov_line}) = LineProjection($self->{vec},$self->{cov});

  return $self;
}

sub ConstructFromARRAY {
  my (@arg) = @_;
  ConstructFromMatrix(mpdl($arg[0]));
}
sub ConstructFromARRAYARRAY {
  my (@arg) = @_;
  ConstructFromMatrixMatrix(mpdl($arg[0]),mpdl($arg[1]));
}
sub ConstructFromARRAYMatrix {
  my (@arg) = @_;
  ConstructFromMatrixMatrix(mpdl($arg[0]),$arg[1]);
}
sub ConstructFromMatrixArray {
  my (@arg) = @_;
  ConstructFromMatrixMatrix($arg[0],mpdl($arg[1]));
}

sub type {
  "Projection";
}

# method return matrix representation of element
sub matrix {
  my ($self,$type) = @_;
  $type ||= "Point"; # default value
  return ($type =~ /point/i) ? $self->{vec}->reshape(3,4) : $self->{vec_line}->reshape(3,6);
}


sub vec {
  my ($self,$type) = @_;
  $type ||= "Point"; # default value
  return ($type =~ /point/i) ? $self->{vec} : $self->{vec_line};
}

sub cov {
  my ($self,$type) = @_;
  $type ||= "Point"; # default value
  return ($type =~ /point/i) ? $self->{cov} : $self->{cov_line};
}

# compute projection matrix for lines
sub LineProjection {
  my ($Pvec,$Pcov) = @_;

  my $r1 = $Pvec->slice("0:3");
  my $r2 = $Pvec->slice("4:7");
  my $r3 = $Pvec->slice("8:11");

  my $rs1 = SUGR::Element::Pidual($r2) x $r3;
  my $rs2 = SUGR::Element::Pidual($r3) x $r1;
  my $rs3 = SUGR::Element::Pidual($r1) x $r2;

  my $Ps = mzeroes(3,6);
  ($_ = $Ps->slice("0:0,0:5")) .= ~$rs1;
  ($_ = $Ps->slice("1:1,0:5")) .= ~$rs2;
  ($_ = $Ps->slice("2:2,0:5")) .= ~$rs3;

  my $J = mzeroes(18,12);
  ($_ = $J->slice("0:5,8:11"))  .= SUGR::Element::Pidual($r2);
  ($_ = $J->slice("6:11,0:3"))  .= SUGR::Element::Pidual($r3);
  ($_ = $J->slice("12:17,4:7")) .= SUGR::Element::Pidual($r1);

  my $Cov = defined($Pcov) ? $J x $Pcov x ~$J : undef;

  # normalize $Ps
  my $factor = 1/(PDL::max(PDL::abs($Ps)));
  $Ps  *= $factor;
  $Cov *= $factor**2;

  return ($Ps->reshape(18,1),$Cov);
}

our $sigma0_projection_factor=1;
sub set_noise_of_projection_center {
#  my ($self,$percentage) = @_;
  my ($self,$standard_deviation) = @_;

  my $KR = $self->matrix->slice("0:2,0:2");
  # this standard deviation holds only for aerial images with a
  # projection center derived from a bundle adjustment. Units are
  # [m]. One should set this from the outside !!!
  $standard_deviation = vpdl([0.075,0.075,0.025]) if !defined($standard_deviation);
  my $variances = ($sigma0_projection_factor*$standard_deviation)**2;
  
  my $X0cov = mzeroes(3,3);
  ($_ = $X0cov->PDL::diagonal(0,1)) .= $variances->PDL::clump(2);

  my $KRX0cov = $KR x $X0cov x ~$KR;

  my $Pvec = $self->vec;
  my $Pcov = $self->cov;
  $Pcov = mzeroes(12,12) if !defined($Pcov);
  
  my @id = (3,7,11);
  foreach my $i (0..2) {
    foreach my $j (0..2) {
      $Pcov->set($id[$i],$id[$j],$KRX0cov->at($i,$j)); 
    }
  } 
#   $percentage ||= 0.0001; # default 
#   my $c = 6793; # focal length in pixel (see Feli)

#   my $Pvec = $self->vec;
#   my $Pcov = $self->cov;
#   $Pcov = mzeroes(12,12) if !defined($Pcov);

#   for (qw/3 7/) {
#     $Pcov->set($_,$_, $c**2 * $percentage**2 * $Pvec->at(11)**2);
#   }
#   $Pcov->set(11,11,$percentage**2 * $Pvec->at(11)**2);

  return sugr_Projection($Pvec,$Pcov);
}

sub euclid_scale {
  return @_
}

sub projection_center {
  my ($self) = @_;
  
  my ($evector,$evalue) = eigens(~$self->matrix x $self->matrix);
  
  my $iev = qsorti $evalue;
  # eigenvector for smallest eigenvalue
  my $sev = $evector->slice(",(".$iev->at(0).")");
  $sev /= $sev->at(3);
  return vpdl($sev);
}

use overload q{""} => \&stringify;

sub stringify { 
  my ($self) = shift;

  (my $s = ref($self)." Matrix::\n") =~ s/SUGR:://;

  foreach my $type (qw/point line/) {
    $s .= "- $type-projection: ".$self->matrix($type);
  }

  my %reshape = ( point => [3,4], line => [3,6] );
  foreach my $type (qw/point line/) {
    if (defined($self->cov($type))) {
      $s .= "Variances of $type-projection matrix: ";
      $s .=  $self->cov($type)->diagonal(0,1)->reshape(@{$reshape{$type}})."\n";
    } 
  }
  return $s;
}

return 1 if caller;

####### Test #########
use PDL::Matrix;

$SUGR::Element::apply_constraints=0;
my $P = sugr_Projection(msequence(3,4),mzeroes(12,12));
my $Pnew = $P->set_noise_of_projection_center(vpdl([0.075,0.075,0.025]));
print $Pnew;
