=head1 NAME

SUGR::RelationPseudoinverse -- relations as described in the DAGM paper 2000 

=head1 VERSION

  This is a deprecated version and should only be used for internal
  testing purposes.

=head1 BUGS

Probably many, but not documented yet.

=head1 SEE ALSO

SUGR(3), SUGR::Relation(3)

=head1 AUTHOR

Stephan Heuel
(stephan@heuel.org)

=head1 COPYRIGHT

Copyright (C) 2000/2001 Stephan Heuel and Institute for Photogrammetry,
University of Bonn. All rights reserved. There is no warranty. You are
allowed to redistribute this software / documentation under certain
conditions, see the GNU Public License, GPL for details.

=cut


package SUGR::RelationPseudoinverse;

use strict;
use Carp;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
$VERSION = 0.01;
@ISA = qw(Exporter);

@EXPORT = qw(); # symbols to autoexport
@EXPORT_OK = qw(); #symbols to export on request
%EXPORT_TAGS = ( );

use PDL;
use PDL::Matrix;
use PDL::Math;
use PDL::Slatec;

use SUGR::Basic qw/:ALL/;

# the overloading of the polymorphic operators works as the
# overloading trick for the constructor. Now I only do this four times
# for the four test types.
foreach my $func qw(Incident Equal Orthogonal Parallel) {
  no strict "refs";
  # define closure, i.e. a function with name $func
  *$func = sub {
    my ($op1,$op2,$op3,$op4) = @_;
    my ($s1,$s2);
    ($s1 = ref($op1)) =~ s/SUGR::(.+)/$1/;
    ($s2 = ref($op2)) =~ s/SUGR::(.+)/$1/;
    $op3 = 0.95 if !defined($op3);
    die "no known SUGR objects given to $func" if ($s1.$s2 eq '');
    my $code = "$func$s1$s2";
    no strict "refs";
    warn "code $code does not exist." if !exists(&$code);
    return () if !exists(&$code);
    return &$code($op1,$op2,$op3,$op4);
  };
  my $op_func = "op_$func";
  *$op_func = sub {
    my ($op1,$op2) = @_;
    my %test = &$func($op1,$op2);
    return $test{bool};
  };
  push @EXPORT,$func;
}

sub EqualPoint2dPoint2d {
  my ($x_in,$y_in,$alpha) = @_;

  my ($factor,$x,$y) = normalize_sugr_objects($x_in,$y_in);

  my $d = $x->S x $y->vec;

  (defined($x->cov) && defined($y->cov)) ?
    1 : return( bool => all((~$d x $d) < eps));

  my $cov = $x->S x $y->cov x ~($x->S) +
            $y->S x $x->cov x ~($y->S);

  my $H = $x->vec;

  my $test = ~$d x pseudoinverse($cov,$H) x $d;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub EqualLine2dLine2d {  my ($x,$y,$alpha) =@_;
  return EqualPoint2dPoint2d($x->dual,$y->dual,$alpha);
}

sub OrthogonalLine2dLine2d {
  my ($l1_in,$l2_in,$alpha) =@_;

  my ($factor,$l1,$l2) = normalize_sugr_objects($l1_in,$l2_in);

  my $d = inner($l1->vec->slice(",(0)"),$l2->vec->slice(",(0)"));

  (defined($l1->cov) && defined($l2->cov)) ?
    1 : return( bool => all((~$d x $d) < eps));

  my $test = $d**2 * 1/(~$l1 x $l2 x $l1 + ~$l2 x $l2 x $l2);

  my $chis=chisqr (1,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub ParallelLine2dLine2d {
  my ($l1_in,$l2_in,$alpha) =@_;

  my ($factor,$l1,$l2) = normalize_sugr_objects($l1_in,$l2_in);

  my $d = Jacobian_ParallelLine2dLine2d($l1,$l2) x $l2;

  (defined($l1->cov) && defined($l2->cov)) ?
    1 : return( bool => all((~$d x $d) < eps));

  my $test = $d**2 * 1/(~$l1 x $l2 x $l1 + ~$l2 x $l2 x $l2);

  my $chis=chisqr (1,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub IncidentPoint2dLine2d {
  my ($x_in,$l_in,$alpha) =@_;

  my ($factor,$x,$l) = normalize_sugr_objects($x_in,$l_in);

  #  my ($x,$l,$factor) = ($x_in,$l_in,1);

  my $d = inner($x->vec->slice(",(0)"),$l->vec->slice(",(0)"));

  (defined($x->cov) && defined($l->cov))
    ? 1 : return( bool => all((~$d x $d) < eps));

  my ($test,$sigma_dd);
  $sigma_dd = (~$x->vec x $l->cov x $x->vec +
	       ~$l->vec x $x->cov x $l->vec);
  $test = $d**2/$sigma_dd;

  my $chis=chisqr(1,$alpha);
  my $dhom = $d * (1/sqrt($l->vec->at(0)**2 + $l->vec->at(1)**2));
  
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0)),
	   distance => $dhom/$factor * (1/($x->vec->at(2)))
	 );
}
sub IncidentLine2dPoint2d { IncidentPoint2dLine2d(@_[1,0,2]) }



#
# 3D tests
#

sub EqualPointPoint {
  my ($X_in,$Y_in,$alpha) =@_;

  my ($factor,$X,$Y) = normalize_sugr_objects($X_in,$Y_in);

  my $idx = ChooseIndex_for_C($X->vec,$Y->vec);
#   my $C1 = C($X->vec,$idx);
#   my $C2 = C($Y->vec,$idx);
  my $C1 = C($X->vec,$Y->vec);
  my $C2 = C($Y->vec,$X->vec);
  my $D = $C1 x $Y->vec;
#   print $idx;
#   print $X,$Y;
#   print $C1,$C2;
#   print $D;

  (defined($X->cov) && defined($Y->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $C1 x $Y->cov x ~$C1 + $C2 x $X->cov x ~$C2;

  my $H = vzeroes(4); # vones(4);
  ($_ = $H->slice("($idx),(0)")) .= 1;

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (3,$alpha);
#  print $chis," > ",$test->at(0,0),"\n";
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub IncidentLinePoint {
  my ($L_in,$X_in,$alpha) = @_;

  my ($factor,$L,$X) = normalize_sugr_objects($L_in,$X_in);

  my $PidualT = ~$X->Pidual;
  my $Gamma = $L->Gamma;
  my $D = $PidualT x $L->vec;

  (defined($X->cov) && defined($L->cov)) ? 
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $PidualT x $L->cov x ~$PidualT + $Gamma x $X->cov x ~$Gamma;

  my $H = ChooseGammaCols($L->Gammadual);


  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}
sub IncidentPointLine { IncidentLinePoint(@_[1,0,2]); }

sub IncidentPointPlane {
  my ($X_in,$A_in,$alpha) =@_;

  my ($factor,$X,$A) = normalize_sugr_objects($X_in,$A_in);

  my $d = inner($X->vec->slice(",(0)"),$A->vec->slice(",(0)"));

  (defined($X->cov) && defined($A->cov)) ? 
    1 : return( bool => ($d*$d < eps));

  my $Cov = $X->vec x $A->cov x ~$X->vec 
    + $A->vec x $X->cov x ~$A->vec;

  my $sigma_dd = (~$X->vec x $A->cov x $X->vec +
		  ~$A->vec x $X->cov x $A->vec);

  my $test = $d**2/$sigma_dd;

  my $chis=chisqr (1,$alpha);

  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub IncidentPlanePoint { IncidentPointPlane(@_[1,0,2]); }

sub EqualLineLine {
  my ($L_in,$M_in,$alpha) =@_;

  my ($factor,$L,$M) = normalize_sugr_objects($L_in,$M_in);

  my $idx = ChooseIndex_for_C($L->vec,$M->vec);

#   my $C1 = C($L->vec,$idx);
#   my $C2 = C($M->vec,$idx);
  my $C1 = C($L->vec,$M->vec);
  my $C2 = C($M->vec,$L->vec);
  my $D = $C1 x $M->vec;
#print $D;

  (defined($L->cov) && defined($M->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $C1 x $L->cov x ~$C1 + $C2 x $M->cov x ~$C2;

  my $H = mzeroes(6,2);
  ($_ = $H->slice("($idx),(0)")) .= 1;
  ($_ = $H->slice(",1")) .= $L->vec;

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (4,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub ParallelLineLine {
  my ($L,$M,$alpha) =@_;

  my $S1 = S($L->vec1);
  my $S2 = S($M->vec1);

  # need normalizing of $S1 and $S2 with their covariances

  my $D = $S1 x $M->vec1;

  (defined($L->cov) && defined($M->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $S1 x $M->cov->slice("0:2,0:2") x ~$S1
    +$S2 x $L->cov->slice("0:2,0:2") x ~$S2;

  my $H = $M->vec1;

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub IncidentLineLine {
  my ($L_in,$M_in,$alpha) =@_;

  my ($factor,$L,$M) = normalize_sugr_objects($L_in,$M_in);

  my $LdT = ~$L->vec->rotate(3);
  my $MdT = ~$M->vec->rotate(3);

  my $D = $LdT x $M->vec;

  (defined($L->cov) && defined($M->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $LdT x $M->cov x ~$LdT +
            $MdT x $L->cov x ~$MdT;
  
  my $test = ($D->at(0,0)**2)/($Cov->at(0,0));

  my $chis=chisqr (1,$alpha);
  return ( bool =>  ($test < $chis),
	   testratio => ($test / $chis),
	   testvalue => sqrt(abs($test)) # absolute value since
                                         # sometimes can be something
                                         # like -1e-30
	 );
}

sub OrthogonalLineLine {
  my ($L,$M,$alpha) =@_;

  my $D = ~$L->vec1 x $M->vec1;

  (defined($L->cov) && defined($M->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = ~$L->vec1 x $M->cov->slice("0:2,0:2") x $L->vec1 +
            ~$M->vec1 x $L->cov->slice("0:2,0:2") x $M->vec1;
  
  my $test = ($D**2)/$Cov;


  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub IncidentLinePlane {
  my ($L_in,$A_in,$alpha) =@_;

  my ($factor,$L,$A) = normalize_sugr_objects($L_in,$A_in);

  my $PiT = ~$A->Pi;
  my $Gammad  = $L->Gammadual;
  my $D = $PiT x $L->vec;

  (defined($L->cov) && defined($A->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $PiT x $L->cov x ~$PiT + $Gammad x $A->cov x ~$Gammad;

  my $H = ChooseGammaCols($L->Gamma);

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  die "IncidentLinePlane: testvalue^2 = $test < 0 !, D= $D, H=$H,Cov=$Cov,pseudoinverse = ".
    (pseudoinverse($Cov,$H))."Line $L, Plane $A, Line IN $L_in, Plane IN $A_in\n" if $test <0;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}
sub IncidentPlaneLine { IncidentLinePlane(@_[1,0,2]); }

sub OrthogonalLinePlane {
  my ($L,$A,$alpha) =@_;

  my $S1 = S($L->vec1);
  my $S2 = S($A->vec->slice("0:2"));

  my $D = $S1 x $A->vec->slice("0:2");

  (defined($L->cov) && defined($A->cov)) ? 
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $S1 x $A->cov->slice("0:2,0:2") x ~$S1 +
            $S2 x $L->cov->slice("0:2,0:2") x ~$S2;

  my $H = mzeroes(3,1);
  ($_ = $H->slice(",0")) .= $L->vec1;

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}
sub OrthogonalPlaneLine {OrthogonalLinePlane(@_[1,0,2])}

sub ParallelLinePlane {
  my ($L,$A,$alpha) =@_;

  my $D = ~$L->vec1 x $A->vec->slice("0:2");

  (defined($L->cov) && defined($A->cov)) ? 
    1 : return( bool => all((~$D x $D) < eps));


  my $Cov = ~$L->vec1 x $A->cov->slice("0:2,0:2") x $L->vec1 +
            ~$A->vec->slice("0:2") x $L->cov->slice("0:2,0:2") x $A->vec->slice("0:2");

  my $test = ~$D x $Cov->matinv x $D;

  my $chis=chisqr (1,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}
sub ParallelPlaneLine {ParallelLinePlane(@_[1,0,2])}


sub EqualPlanePlane {
  my ($A,$B,$alpha) =@_;
#  return EqualPointPoint($A->dual,$B->dual,$alpha);
  return EqualPointPoint($A,$B,$alpha);
}

sub ParallelPlanePlane {
  my ($A,$B,$alpha) =@_;

  my $SA = S($A->vec->slice("0:2"));
  my $SB = S($B->vec->slice("0:2"));

  my $D = $SA x $B->vec->slice("0:2");

  (defined($A->cov) && defined($B->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = $SA x $B->cov->slice("0:2,0:2") x ~$SA +
            $SB x $A->cov->slice("0:2,0:2") x ~$SB;

  my $H = mzeroes(3,1);
  ($_ = $H->slice(",0")) .= $A->vec->slice("0:2");

  my $test = ~$D x pseudoinverse($Cov,$H) x $D;

  my $chis=chisqr (2,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub OrthogonalPlanePlane {
  my ($A,$B,$alpha) =@_;

  my $D =  ~$A->vec->slice("0:2") x $B->vec->slice("0:2");

  (defined($A->cov) && defined($B->cov)) ?
    1 : return( bool => all((~$D x $D) < eps));

  my $Cov = ~$A->vec->slice("0:2") x $B->cov->slice("0:2,0:2") x $A->vec->slice("0:2") +
            ~$B->vec->slice("0:2") x $A->cov->slice("0:2,0:2") x $B->vec->slice("0:2");

  my $test = ~$D x $Cov->matinv x $D;

  my $chis=chisqr (1,$alpha);
  return ( bool => ($test->at(0,0)< $chis),
	   testratio => ($test->at(0,0)/ $chis),
	   testvalue => sqrt($test->at(0,0))
	 );
}

sub ChooseGammaCols {
  my ($Gammadual) = @_;

  my $vec = $Gammadual->abs->transpose->sumover;

  my $i1 = int $vec->maximum_ind;
  set($vec,$i1,0);
  my $i2 = int $vec->maximum_ind;
  return $Gammadual->dice([0..3],[$i1,$i2]);
}

return 1 if caller;

# Test code

use SUGR::Point2d;
use SUGR::Line2d;
use PDL::Matrix;

# apply_constraints(0);
# my $x_empty = sugr_Point2d(vsequence(3),Identity(3));
# my $l_empty = sugr_Line2d (vsequence(3),Identity(3));

# my $x_euclid = sugr_Point2d(vsequence(3),Identity(3))->euclidize;
# my $l_euclid = sugr_Line2d (vsequence(3),Identity(3))->euclidize;

# apply_constraints(1);
# my $x_sphere = sugr_Point2d(vsequence(3),Identity(3));
# my $l_sphere = sugr_Line2d (vsequence(3),Identity(3));

# print "nullspace = euclidean : ",Incident($x_euclid,$l_euclid),"\n";
# print "nullspace = sphere    : ",Incident($x_sphere,$l_sphere),"\n";
# print "nullspace = empty     : ",Incident($x_empty,$l_empty),"\n";

# test all 3D relations
use SUGR::Cuboid;

my %Objects = generate_cuboid(no_points => 10,
			      no_planes => 10,
			      no_lines  => 10,
			      random_seed => [4711,222712]);
 %Objects = transform_cuboid(translation => [1,2,2],
 			    rotation    => [0.5,0.7,1.3],
 			    %Objects);

my $X_cube  = $Objects{Points};
my $L_cube  = $Objects{Lines};
my $A_cube  = $Objects{Planes};
my $X_cube_ideal  = $Objects{Ideal_Points};
my $L_cube_ideal  = $Objects{Ideal_Lines};
my $A_cube_ideal  = $Objects{Ideal_Planes};

my %Testobjects = ( 
   		   "EqualPointPoint"    => [$X_cube_ideal->[0],$X_cube->[0]],
  		   "EqualLineLine"      => [$L_cube_ideal->[0],$L_cube->[0]],
  		   "EqualPlanePlane"    => [$A_cube_ideal->[0],$A_cube->[0]],
  		   "IncidentPointLine"  => [$X_cube_ideal->[0],$L_cube->[0]],
  		   "IncidentPointPlane" => [$X_cube_ideal->[0],$A_cube->[0]],
  		   "IncidentLineLine"   => [$L_cube_ideal->[0],$L_cube->[1]],
  		   "IncidentLinePlane"  => [$L_cube_ideal->[0],$A_cube->[0]],
  		   "OrthogonalLineLine"   => [$L_cube_ideal->[0],$L_cube->[1]],
  		   "OrthogonalLinePlane"  => [$L_cube_ideal->[0],$A_cube->[2]],
  		   "OrthogonalPlanePlane" => [$A_cube_ideal->[0],$A_cube->[1]],
  		   "ParallelLineLine"     => [$L_cube_ideal->[0],$L_cube->[2]],
  		   "ParallelLinePlane"    => [$L_cube_ideal->[0],$A_cube->[3]],
  		   "ParallelPlanePlane"   => [$A_cube_ideal->[0],$A_cube->[5]],
      	   
 		   "NotIncidentLineLine"   => [$L_cube_ideal->[0],$L_cube->[6]],
		   "NotEqualPointPoint"    => [$X_cube_ideal->[0],$X_cube->[2]],
       	           "NotEqualLineLine"      => [$L_cube_ideal->[0],$L_cube->[2]],
		   "NotEqualPlanePlane"    => [$A_cube_ideal->[0],$A_cube->[3]],

		  );


foreach my $Test (keys %Testobjects) {
  print "$Test\n",map { "-" } (1..length($Test));
  print "\n";

  my @O1 = @{$Testobjects{$Test}->[0]};
  my @O2 = @{$Testobjects{$Test}->[1]};

  my $wrong = ($Test =~ s/^Not//) ? 1 : 0;
  
  foreach my $O1 (@O1) {
    foreach my $O2 (@O2) {
      no strict "refs";
      my %Result = &$Test($O1,$O2,0.99);
      die "Test was rejected @{[ %Result ]}" if ((!$Result{bool} && !$wrong) ||
						 ($Result{bool} && $wrong));
      print "Test successful: testvalue $Result{testvalue}\n";
    }
  }
}
