# -*- cperl -*-

# test if the code mentioned in the synopsis of SUGR.pm works.

#use strict;
use Test;

BEGIN { $| = 1; plan tests => 8 }

use SUGR;

# create a new 3D point 
my $X = new SUGR::Point([0,1,1,1],             # homogeneous vector,
   			[[0.01,0   ,0   ,0],        # homogeneous 
   			 [0   ,0.01,0   ,0],   # covariance matrix
   			 [0   ,0   ,0.01,0],
   			 [0   ,0   ,0   ,0]]);
ok(ref($X),'SUGR::Point');

# create two other points with 
# the same covariance matrix as the first point
my $Y = new SUGR::Point([0,2,3,1],$X->cov); 
ok(ref($Y),'SUGR::Point');
my $Z = new SUGR::Point([2,2,0,1],$X->cov); 
ok(ref($Z),'SUGR::Point');

# construct a new 3D line using $X and $Y, 
# including error propagation
my $L = new SUGR::Line($X,$Y);
ok(ref($L),'SUGR::Line');

# and a plane with the third point $Z
my $A = new SUGR::Plane($L,$Z);
ok(ref($A),'SUGR::Plane');

# create a projective camera $P with given projection center
# $X0 (as a SUGR point!), rotation matrix $R and 
# focal length $c with its variance $c_var
my $X0 = new SUGR::Point([10,10,10,1],$X->cov); 
my $R  = new PDL::Matrix([[0,1,0],[1,0,0],[0,0,1]]);
ok(ref($R),'PDL::Matrix');
my $P  = sugr_ProjectiveCamera({ "X0"    => $X0,
   				 "R"     => $R,
   				 "c"     => 1,
   				 "c_var" => 0.001,
   			       });
ok(ref($P),'SUGR::ProjectiveCamera');

# construct a new 2D point as an image of $X with 
# the projective camera $P
$x = new SUGR::Point2d($X,$P);
ok(ref($x),'SUGR::Point2d');
