;; (c) by IPB, Bonn
;;
;; This is a major mode for GXF files in XEmacs
;; Version 0.1
;;
;; $Docid: Jul. 27th 1998  15:06  Stephan Heuel $
;; $Contactid: heuel@ipb.uni-bonn.de $
;;

;; Create mode-specific tables.
(defvar gxf-mode-syntax-table nil
  "Syntax table used while in text mode.")

(if gxf-mode-syntax-table
         ()              ; Do not change the table if it is already set up, is somehow useless with
					; the above defvar
       (setq gxf-mode-syntax-table (make-syntax-table))
       (modify-syntax-entry ?\" ".   " gxf-mode-syntax-table))

(defvar gxf-mode-map nil)   ; Create a mode-specific keymap.

(defvar gxf-tab-width 4
  "tab-width for gxf-mode")

(if gxf-mode-map
    ()              ; Do not change the keymap if it is already set up, is somehow useless with
					; the above defvar
  (setq gxf-mode-map (make-sparse-keymap))
  (define-key gxf-mode-map "\t" 'tab-to-tab-stop))

(defvar gxf-mode-abbrev-table nil
  "Abbrev table used while in gxf mode.")
(define-abbrev-table 'gxf-mode-abbrev-table ())


;; things for font-lock
;; the regexps must be optimized
(defconst gxf-font-lock-keywords
  '(("^[ \t\r]*\\(BLOCKSTART\\)[ \t\r]+\\([a-zA-Z_0-9]*\\)"
     (1 font-lock-function-name-face) (2 font-lock-variable-name-face nil t))
    ("^[ \t\r]*\\(BLOCKEND\\)[ \t\r]+\\([a-zA-Z_0-9]*\\)"
     (1 font-lock-function-name-face) (2 font-lock-variable-name-face nil t))
    ("^[ \t\r]*\\(ELEMENTLIST\\)[ \t\r]+\\([a-zA-Z_0-9]*\\)"
     (1 font-lock-keyword-face) (2 font-lock-variable-name-face nil t))
    ("^[ \t\r]*\\(ELEMENT\\)[ \t\r]+\\([a-zA-Z_0-9]*\\)"
     (1 font-lock-function-name-face) (2 font-lock-variable-name-face nil t))
    ("^[ \t\r]*\\(END_ELEMENT\\)"
     1 font-lock-function-name-face)
    ("^[ \t\r]*\\(LIST_OF\\)[ \t\r]+\\([a-zA-Z_0-9]*\\)"
     (1 font-lock-keyword-face) (2 font-lock-variable-name-face nil t))
    ("^[ \t\r]*\\(END_OF_FILE\\)"
     1 font-lock-function-name-face)
    ("\\(;.*\\)$" 1 font-lock-comment-face)
    ("\\(\"[^\"]*\"\\)" 1 font-lock-string-face))
  "Expressions to highlight in GXF mode.")
(put 'gxf-mode 'font-lock-defaults '(gxf-font-lock-keywords))

(defun gxf-mode ()
  "Major mode for Generic Exchange Format GXF
      Special commands: \\{gxf-mode-map}

     Turning on gxf-mode runs the hook `gxf-mode-hook'."

  (interactive)
  (kill-all-local-variables)
  
  (use-local-map gxf-mode-map)     ; This provides the local keymap.
  (setq mode-name "GXF")           ; This name goes into the modeline.
  (setq major-mode 'gxf-mode)      ; This is how `describe-mode'
					;   finds the doc string to print.
  (setq local-abbrev-table gxf-mode-abbrev-table)
  (set-syntax-table gxf-mode-syntax-table)

  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(gxf-font-lock-keywords))

  (setq tab-width gxf-tab-width)

  (run-hooks 'gxf-mode-hook))      ; Finally, this permits the user to
					;   customize the mode with a hook.

;; invoke gxf-mode automatically when loading *.gxf files
(setq auto-mode-alist  (cons  '("\\.gxf\\'" . gxf-mode) auto-mode-alist))

(provide 'gxf-mode)