#!/usr/bin/perl

use strict;
use warnings;

#
# Quick hack for installing the SUGR bundle including all required
# modules. This has to change in future using CPAN and autobundle
# method, but currently, not all modules are on CPAN and we use a
# patched version of PDL.
#
# See also INSTALL and README files
#

#
# Here you can change the prefix of the directories, in which the perl
# modules are installed. It is recommended to start the script as root
# when installing so that these directories are chosen by the system
# automatically.
#
# my $PREFIX = "/usr/local/";
#

my $PREFIX = $ENV{HOME};


####
#### You shouldn't change anything after this point
####

my %Archives = (
		1 => "ExtUtils-F77-1.14",
		2 => "Math-Random-0.63",
		3 => "PDL-2.3.2",
		4 => "Storable-1.0.13",
		5 => "Statistics-GaussHelmert-0.02",
		6 => "SUGR-0.1.0",
	       );


my $install_user = <<EOT;
	perl Makefile.PL LIB=$PREFIX/perllib PREFIX=$PREFIX
	make 
	make test
	make install
EOT

my $install_root = <<EOT;
	perl Makefile.PL
	make 
	make test
	make install
EOT

my $rootp = `whoami` eq "root";

my $install = $rootp ? $install_root : $install_user;

print <<EOT;

SUGR
----

Automatic Installation of the SUGR library with its
dependent modules.

See the README and INSTALL for details.

(c) Stephan Heuel and Institute for Photogrammetry, Bonn
    heuel\@ipb.uni-bonn.de

ATTENTION: The compilation and installation may take a couple of
minutes.

EOT

unless ($rootp) {
  print "User Installation configuration:\n";
  print "PREFIX = $PREFIX\n\n";
  
  die "*** Error: it seems that \$PERL5LIB is not defined, see INSTALL file.\n"
    unless defined($ENV{PERL5LIB});
  die "*** Error: it seems that \$PERL5LIB does not contain $PREFIX/perllib\n"
    unless $ENV{PERL5LIB} =~ m!$PREFIX/perllib!;
  die "*** Error: it seems that \$MANPATH does not contain $PREFIX/man\n"
    unless $ENV{MANPATH} =~ m!$PREFIX/man!;
  die "*** Error: it seems that \$PATH does not contain $PREFIX/bin\n"
    unless $ENV{PATH} =~ m!$PREFIX/bin!;
}
else {
  print "You have root privileges.\nPerl-Software will be automatically installed for your system.\n\n"
}
print "Press RETURN to start";

<STDIN>;

# check directories
foreach (qw! perllib man man/man1 man/man3 bin!) {
  mkdir "$PREFIX/$_" or die "cannot create $PREFIX/$_: $!" unless -e "$PREFIX/$_";
}


# final installation
foreach (sort keys %Archives) {
  my $archive = $Archives{$_};

  print "\n$archive\n",(map { "-" } (1..length($archive))),"\n";

  (my $dir = "./$archive") =~ s/-patched//;

  if (!-e $dir ) {
    system("tar xzf $archive.tar.gz") == 0 or die "*** Error: cannor untar $archive:\n$!"; 
  }
  chdir("$dir") or die "*** Error: cannot change to $dir\n$!";
  foreach my $line (split("\n",$install)) {
    next if $line =~ /Makefile.PL/ && -e "./Makefile";
    system("$line") == 0 
      or die "*** Error: Problems when doing $line in $archive:\n$!";
  }
  chdir("../")  or die "*** Error: cannot change back to ../\n$!";;
}

print <<EOT;

Done with the installation.

Try 

  sugr SUGR-0.1.0/examples/scene.gxf 

to read a predefined scene and do some basic reasoning, see the manual
pages SUGR(3), sugr(1) and sugrshell(1) for details.

Have fun.

EOT

